package com.arubanetworks.meridiansamples;

import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.PointF;
import android.graphics.RectF;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import com.arubanetworks.meridian.location.MeridianLocation;
import com.arubanetworks.meridian.location.MeridianOrientation;
import com.arubanetworks.meridian.maps.MapInfo;
import com.arubanetworks.meridian.requests.MapInfoRequest;
import com.arubanetworks.meridian.requests.MeridianRequest;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.PolygonOptions;

/**
 * A Fragment to demonstrate the integration with Google maps.  NOTE: this sample requires a Google maps API key which needs
 * to be set in the manifest (android:value="ADD_YOUR_GOOGLE_MAPS_API_KEY_HERE").
 */
public class GoogleMapsFragment extends Fragment implements OnMapReadyCallback, com.arubanetworks.meridian.maps.MapView.MapEventListener {
    private MapView mGoogleMapView;
    private com.arubanetworks.meridian.maps.MapView mMeridianMapView;
    private GoogleMap mGoogleMap;
    private MapInfo mMapInfo;
    private float mGoogleMapHeading;
    private boolean meridianMapLoaded = false;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        new MapInfoRequest.Builder()
                .setMapKey(Application.MAP_KEY)
                .setListener(new MeridianRequest.Listener<MapInfo>() {
                    @Override
                    public void onResponse(MapInfo mapInfo) {
                        mMapInfo = mapInfo;
                        addMapOverlay();
                    }
                })
                .setErrorListener(new MeridianRequest.ErrorListener() {
                    @Override
                    public void onError(Throwable tr) {
                        // TODO do something
                    }
                })
                .build()
                .sendRequest();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View layout = inflater.inflate(R.layout.fragment_google_maps, container, false);
        mGoogleMapView = layout.findViewById(R.id.gm_googlemap);
        // Note that the Google map was preloaded in MainActivity because this can take 3+ seconds
        mGoogleMapView.onCreate(savedInstanceState);
        mGoogleMapView.getMapAsync(this);
        //setup meridian map
        mMeridianMapView = layout.findViewById(R.id.gm_meridian_map);
        mMeridianMapView.setAppKey(Application.APP_KEY);
        mMeridianMapView.setMapKey(Application.MAP_KEY);
        mMeridianMapView.setMapEventListener(this);
        return layout;
    }

    @Override
    public void onStart() {
        super.onStart();
        meridianMapLoaded = false;
        mGoogleMapView.onStart();
    }

    @Override
    public void onResume() {
        super.onResume();
        mGoogleMapView.onResume();
        mMeridianMapView.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
        mGoogleMapView.onPause();
        mMeridianMapView.onPause();
    }

    @Override
    public void onStop() {
        super.onStop();
        mGoogleMapView.onStop();
    }

    @Override
    public void onLowMemory() {
        super.onLowMemory();
        mGoogleMapView.onLowMemory();
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle bundle) {
        super.onSaveInstanceState(bundle);
        mGoogleMapView.onSaveInstanceState(bundle);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mGoogleMapView.onDestroy();
        mMeridianMapView.onDestroy();
        mMeridianMapView.setMapEventListener(null);
    }

    @Override
    public void onMapReady(GoogleMap map) {
        mGoogleMap = map;
        try {
            map.setMyLocationEnabled(true);
        } catch (SecurityException ignore){}
        addMapOverlay();
        mGoogleMapHeading = (float) Math.toRadians(mGoogleMap.getCameraPosition().bearing);
        mGoogleMap.setOnCameraIdleListener(new GoogleMap.OnCameraIdleListener() {
            @Override
            public void onCameraIdle() {
                if (mGoogleMapView.getVisibility() == View.VISIBLE) {
                    CameraPosition cameraPosition = mGoogleMap.getCameraPosition();
                    if (cameraPosition.zoom > 18.0) {
                        mGoogleMapHeading = (float) Math.toRadians(cameraPosition.bearing);
                        if (meridianMapLoaded) {
                            resetMeridianMap();
                        }
                        mGoogleMapView.setVisibility(View.GONE);
                        mMeridianMapView.setVisibility(View.VISIBLE);
                        // zoom out a bit to smooth transition
                        mGoogleMap.animateCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(cameraPosition.target.latitude,
                                cameraPosition.target.longitude), 17.0f));
                    }
                }
            }
        });
    }

    private void addMapOverlay() {
        if (mMapInfo != null && mGoogleMap != null) {
            PointF topLeftPoint = mMapInfo.mapPointLatLong(0f,0f);
            PointF topRightPoint = mMapInfo.mapPointLatLong((float) mMapInfo.getWidth(),0);
            PointF bottomLeftPoint = mMapInfo.mapPointLatLong(0,(float)mMapInfo.getHeight());
            PointF bottomRightPoint = mMapInfo.mapPointLatLong((float) mMapInfo.getWidth(),(float)mMapInfo.getHeight());
            LatLng topLeftLatLong = new LatLng(topLeftPoint.x, topLeftPoint.y);
            LatLng topRightLatLong = new LatLng(topRightPoint.x, topRightPoint.y);
            LatLng bottomLeftLatLong = new LatLng(bottomLeftPoint.x, bottomLeftPoint.y);
            LatLng bottomRightLatLong = new LatLng(bottomRightPoint.x, bottomRightPoint.y);
            mGoogleMap.addPolygon(new PolygonOptions()
                    .add(topLeftLatLong, topRightLatLong, bottomRightLatLong, bottomLeftLatLong)
                    .strokeColor(Color.RED)
                    .fillColor(Color.BLUE));
            mGoogleMap.moveCamera(CameraUpdateFactory.newLatLngZoom(topLeftLatLong,16));
        }
    }

    private void resetMeridianMap() {
        // rotate map to match Google map orientation and zoom out
        float rotation = (float)(2 * Math.PI - (mMeridianMapView.getMapInfo().getNorth() + mGoogleMapHeading));
        mMeridianMapView.scrollToRect(new RectF(0,0,(float)mMapInfo.getWidth(),(float)mMapInfo.getHeight()), rotation,false);
    }

    //
    // MapViewListener methods
    //
    @Override public void onMapTransformChange(Matrix transform) {
        if (mMeridianMapView.getVisibility() == View.VISIBLE && meridianMapLoaded) {
            // get current zoom scale value from transform
            float[] transformMatrix = new float[9];
            transform.getValues(transformMatrix);
            // Compensate scale for rotation
            float mapScale = (float) Math.sqrt(transformMatrix[Matrix.MSCALE_X] * transformMatrix[Matrix.MSCALE_X] +
                    transformMatrix[Matrix.MSKEW_Y] * transformMatrix[Matrix.MSKEW_Y]);

            // Minimum zoom scale required to fit the entire map on screen
            double minimumZoomScale = Math.min(mMeridianMapView.getWidth() / mMapInfo.getWidth(),
                    mMeridianMapView.getHeight() / mMapInfo.getHeight());
            // Dismiss the meridian map if we zoom out much farther than that.
            if (minimumZoomScale * 0.9 > mapScale) {
                mGoogleMapView.setVisibility(View.VISIBLE);
                mMeridianMapView.setVisibility(View.GONE);
                resetMeridianMap();
            }
        }
    }
    @Override public void onMapLoadStart() {meridianMapLoaded = false; }
    @Override public void onMapLoadFinish() {
        mMapInfo = mMeridianMapView.getMapInfo();
        meridianMapLoaded = true;
        resetMeridianMap();
    }
    @Override public void onPlacemarksLoadFinish() {}
    @Override public void onMapLoadFail(Throwable tr) { }
    @Override public void onMapRenderFinish() { }
    @Override public void onLocationUpdated(MeridianLocation location) { }
    @Override public void onOrientationUpdated(MeridianOrientation orientation) { }
    @Override public boolean onLocationButtonClick() { return false; }
    @Override public boolean onLongPress() { return false; }
}
